#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/fb.h>
#include <linux/init.h>

#include <asm/uaccess.h>

#define SF_WIDTH 240
#define SF_HEIGHT 320
#define SF_BPP 3 /* bytes per pixel */
#define SF_STRIDE 1024
#define SF_BASE  0xf8200000

static struct fb_ops sf_ops = {
    .fb_check_var   = NULL, /* no check: we are lazy */
    .fb_set_par     = NULL, /* no set par: we are lazy */
    .fb_setcolreg   = NULL, /* no colormaps */
    .fb_pan_display = NULL, /* no pan support */
    .fb_mmap        = NULL /*sf_mmap*/,

    /* these are mandatory and implemented elsewhere */
    .fb_fillrect    = cfb_fillrect,
    .fb_copyarea    = cfb_copyarea,
    .fb_imageblit   = cfb_imageblit,

    /* we have no cursor support */
    .fb_cursor      = soft_cursor,
};


struct fb_var_screeninfo sf_var __initdata = {
    .xres = SF_WIDTH,
    .yres = SF_HEIGHT,
    .xres_virtual = SF_WIDTH,
    .yres_virtual = SF_HEIGHT,
    .red   = {16,8,},
    .green = { 8,8,},
    .blue  = { 0,8,},
    .bits_per_pixel = 8 * SF_BPP,
    .vmode = FB_VMODE_NONINTERLACED,
};


struct fb_fix_screeninfo sf_fix __initdata = {
    .id = "small-FB",
    .type =  FB_TYPE_PACKED_PIXELS,
    .visual = FB_VISUAL_TRUECOLOR,
    .line_length = SF_STRIDE * SF_BPP,
    .smem_start = SF_BASE,
    .smem_len = SF_HEIGHT * SF_STRIDE * SF_BPP,
    .accel = FB_ACCEL_NONE,
};


/* description of this framebuffer */
struct fb_info sf_info = {
    .fbops = &sf_ops,
    .screen_base = 0 /* to be compiled at load time */,
    .screen_size = SF_STRIDE * SF_HEIGHT * SF_BPP,
    .state = FBINFO_STATE_RUNNING,
};

/* init and cleanup */
int __init sf_init(void)
{
    int ret;
    char __iomem *addr;

    sf_info.fix = sf_fix;
    sf_info.var = sf_var;
    addr = ioremap(SF_BASE, sf_info.screen_size);
    if (!addr) return -ENOMEM;
    sf_info.screen_base = addr;
    ret = register_framebuffer(&sf_info);
    if (ret)
	iounmap(addr);
    else
	printk("small-FB registered\n");
    return ret;
}

void __exit sf_exit(void)
{
    unregister_framebuffer(&sf_info);
    iounmap(sf_info.screen_base);
    printk("small-FB unregistered\n");
}

module_init(sf_init);
module_exit(sf_exit);
